<?php

declare(strict_types=1);
namespace Drupal\translation_bliss_config_language_select\Hooks;

use Drupal\Core\Config\Entity\ConfigEntityInterface;
use Drupal\Core\Config\Entity\ConfigEntityStorageInterface;
use Drupal\Core\Config\TypedConfigManagerInterface;
use Drupal\Core\DependencyInjection\AutowireTrait;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\EntityFormInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\ConfigTarget;
use Drupal\Core\Form\ConfirmFormInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Language\LanguageInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Render\Element;
use Drupal\hux\Attribute\Alter;

final class ConfigLanguageSelectorHooks implements ContainerInjectionInterface {

  use AutowireTrait;

  private const ExcludedFormIds = [
    // Langcode is used for custom language here.
    'language_admin_add_form',
  ];

  public function __construct(
    private readonly EntityTypeManagerInterface $entityTypeManager,
    private readonly LanguageManagerInterface $languageManager,
    private readonly TypedConfigManagerInterface $typedConfigManager,
  ) {}

  // Hux coughs on long module names.
  // https://www.drupal.org/project/hux/issues/3472641
  // #[Alter('form')]
  function hookFormAlter(array &$form, FormStateInterface $form_state, string $form_id): void {
    if (in_array($form_id, self::ExcludedFormIds)) {
      return;
    }

    $formObject = $form_state->getFormObject();
    if ($formObject instanceof ConfirmFormInterface) {
      return;
    }

    if ($formObject instanceof ConfigFormBase) {
      $this->addSelectorToConfigForm($form, $formObject);
    }
    elseif ($formObject instanceof EntityFormInterface) {
      $entity = $formObject->getEntity();
      if ($entity instanceof ConfigEntityInterface) {
        $entity = $this->loadOverrideFree($entity);
        $form = $this->addSelectorToConfigEntityForm($form, $entity);
      }
    }
  }

  /**
   * Add the config language selector to config forms.
   */
  function addSelectorToConfigForm(array &$form, ConfigFormBase $formObject): void {
    if (!isset($form['langcode'])) {
      // Get the element / config map.
      $map = [];
      $this->collectConfigTargetMap($form, $map);

      $configNames = array_keys($map);
      $multiConfig = count($configNames) > 1;
      foreach ($configNames as $configName) {
        $typedConfig = $this->typedConfigManager->get($configName);
        $configLabel = $typedConfig->getDataDefinition()->getLabel();

        if (!isset($map[$configName]['langcode'])) {
          // The value is picked up by ConfigFormBase.
          /** \Drupal\Core\Form\ConfigFormBase::copyFormValuesToConfig */

          $key = sprintf("langcode__%s", str_replace('.', '__', $configName));
          $form[$key] = [
            '#config_target' => "{$configName}:langcode",
            '#title' => $multiConfig ?
              t('Configuration language for %config', ['%config' => $configLabel]) :
              t('Configuration language'),
            '#description' => t('Changes the configuration language. Does not translate any strings.'),
            '#type' => 'language_select',
            // @todo Add configurable whitelist.
            '#languages' => LanguageInterface::STATE_CONFIGURABLE,
            '#weight' => -100,
            // No need for #default_value:
            /** @see \Drupal\Core\Form\ConfigFormBase::loadDefaultValuesFromConfig */
          ];
        }
      }
    }
  }

  /**
   * A lightweight version of config target extraction.
   *
   * No element parents, no duplicate detection.
   *
   * @see \Drupal\Core\Form\ConfigFormBase::doStoreConfigMap
   */
  private function collectConfigTargetMap(array $element, array &$map): void {
    if ($target = $element['#config_target'] ?? NULL) {
      if (is_string($target)) {
        $target = ConfigTarget::fromString($target);
      }
      foreach ($target->propertyPaths as $propertyPath) {
        $map[$target->configName][$propertyPath] = TRUE;
      }
    }
    foreach (Element::children($element) as $key) {
      $this->collectConfigTargetMap($element[$key], $map);
    }

  }

  /**
   * Add the config language selector to config entity forms.
   */
  public function addSelectorToConfigEntityForm(array $form, ConfigEntityInterface $entity): array {
    if (!isset($form['langcode'])) {
      $languages = $this->languageManager->getLanguages();
      $defaultLanguage = $this->languageManager->getDefaultLanguage();
      // Favor 'en' over defaultLanguage for config creation.
      // @todo Make configurable.
      $defaultLangcode = isset($languages['en']) ? 'en' : $defaultLanguage->getId();

      // The value is - if not overridden - picked up by EntityForm.
      /** @see \Drupal\Core\Entity\EntityForm::copyFormValuesToEntity */
      // More about language selector:
      /** @see \Drupal\Core\Render\Element\LanguageSelect */
      /** @see \language_element_info_alter() */
      /** @see \language_process_language_select() */
      $form['langcode'] = [
        '#title' => t('Configuration language'),
        '#description' => t('Changes the configuration language. Does not translate any strings.'),
        '#type' => 'language_select',
        // @todo Add configurable whitelist.
        '#languages' => LanguageInterface::STATE_CONFIGURABLE,
        '#default_value' => $entity->get('langcode') ?? $defaultLangcode,
        '#weight' => -100,
      ];
    }
    return $form;
  }

  /**
   * Get an override free version of the config entity.
   *
   * While config has immutable / editable, config entities do not.
   * Entities from the route controller are loaded override-free on admin theme
   * pages, but this is a hack and there are cases where config entities are
   * loaded in another way. Example: EntityViewDisplay.
   *
   * @link https://drunkenmonkey.at/blog/config_entity_overrides
   */
  function loadOverrideFree(ConfigEntityInterface $entity): ConfigEntityInterface {
    $storage = $this->entityTypeManager->getStorage($entity->getEntityTypeId());
    // This should always be the case.
    if ($storage instanceof ConfigEntityStorageInterface) {
      if ($entity->id()) {
        $entity = $storage->loadOverrideFree($entity->id())
          ?? $entity;
      }
    }
    return $entity;
  }

}
